﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Numerics;

namespace CatShooting
{
    public partial class Form1 : Form
    {
        Ship ship;
        List<Bullet> bullets = new List<Bullet>();
        List<Ball> balls = new List<Ball>();
        int count = 30;
        int score = 0;

        Timer ballGenTimer;
        Timer countTimer;

        public Form1()
        {
            InitializeComponent();

            this.ship = new Ship();
            this.Size = new Size(1080, 720);

            Timer timer = new Timer();
            timer.Interval = 16;
            timer.Tick += new EventHandler(Update);
            timer.Start();

            this.ballGenTimer = new Timer();
            this.ballGenTimer.Interval = 1000;
            this.ballGenTimer.Tick += new EventHandler(GenerateBall);
            this.ballGenTimer.Start();

            this.countTimer = new Timer();
            this.countTimer.Interval = 1000;
            this.countTimer.Tick += new EventHandler(Countdown);
            this.countTimer.Start();
        }
        private void Countdown(object sender, EventArgs e)
        {
            this.count--;
            this.timerLabel.Text = count.ToString("D2");

            if (this.count == 0)
            {
                this.ballGenTimer.Stop();
                this.countTimer.Stop();
            }
        }

        private void GenerateBall(object sender, EventArgs e)
        {
            Random r = new System.Random();
            Ball ball;
            int dice = r.Next(1, 10);
            ball = (dice <= 2) ? new IronBall() : new Ball();

            ball.pos.X = this.Size.Width;
            ball.pos.Y = r.Next(100, this.Size.Height - 100);
            this.balls.Add(ball);
        }

        private void CheckCollision()
        {
            foreach (Ball ball in this.balls)
            {
                foreach (Bullet bullet in this.bullets)
                {
                    Vector2 ballPos = ball.pos;
                    Vector2 bulletPos = bullet.pos;
                    float len = (ballPos - bulletPos).Length();

                    if (len <= ball.size.X / 2 + bullet.radius)
                    {
                        bullet.willDestroy = true;
                        ball.Hit();

                        if (ball.GetType() == typeof(Ball))
                        {
                            this.score += 50;
                            this.scoreLabel.Text = score.ToString("D5");
                        }
                    }
                }
            }

            foreach (Ball ball in this.balls)
            {
                if (ball.GetType() != typeof(IronBall)) continue;

                Vector2 ballPos = ball.pos;
                Vector2 shipPos = ship.pos;
                float len = (ballPos - shipPos).Length();

                if (len <= ball.size.X / 2 + ship.size.X / 2)
                {
                    this.ship.Stop();
                }
            }
        }
        private void Update(object sender, EventArgs e)
        {
            Point cp = PointToClient(Cursor.Position);
            this.ship.Update(cp.Y);

            foreach (Bullet b in this.bullets)
            {
                b.Update();
            }
            foreach (Ball b in this.balls)
            {
                b.Update();
            }

            CheckCollision();

            this.bullets.RemoveAll(b => b.willDestroy);
            this.balls.RemoveAll(b => b.willDestroy);

            Invalidate();
        }

        private void Draw(object sender, PaintEventArgs e)
        {
            foreach (Bullet b in this.bullets)
            {
                b.Draw(e);
            }
            foreach (Ball b in this.balls)
            {
                b.Draw(e);
            }

            this.ship.Draw(e);
        }

        private void Shoot(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Space)
            {
                Bullet b = new Bullet();
                b.pos = this.ship.pos;
                this.bullets.Add(b);
            }
        }
        
    }
}
